/* collect.js
 * Collect various information about the application into an in-memory XML
 * Requires: common.js
 *
 * Part of InfoLister by Nickolay Ponomarev <asqueella@gmail.com>
 * Copyright (c) 2004-2005, Nickolay Ponomarev.
 * See the license.txt included in this package for licensing information.
 */

var ILInfoCollector = {
  // The only public method: collectData()
  // Collects data (as specified in aPrefs), return an XML document
  collectData: function(aPrefs) {
    var doc = this.dataDoc = document.implementation.createDocument("","",null);

    // root element, <info app="Whatever" />
    var info = this.infoElt = doc.createElement("info");
    info.setAttribute("app", this.getAppName());
    doc.appendChild(info);
    this.insertLinebreak(info);
    //e4x: doc = <info app="{1+2}"></info>

    // other information
    var checkList = {
      lastupd:   "this.appendNode('lastupd')",
      ua:        "this.appendNode('useragent')",
      extension: "this.appendAddonsList('extension')",
      theme:     "this.appendAddonsList('theme')",
      plugin:    "this.appendPlugins()"
    };
    var xml = InfoLister.mode == 2;
    for(var i in checkList) {
      if(!xml || aPrefs.getBoolPref(i + "bool")) // put everything when not in xml mode (xxx temporary fix)
        eval(checkList[i]);
    }
    return doc;
  },

  /**** DOM helpers ***********************************************************/
  // return a node of form: <aElemName>aData</aElemName>
  makeNodeWithText: function(aElemName, aData) {
    var dataNode = this.dataDoc.createElement(aElemName);
    var textNode = this.dataDoc.createTextNode(aData);
    dataNode.appendChild(textNode);
    return dataNode;
  },

  // insert a line break, so that the serialized file looks nice
  insertLinebreak: function(aRootNode) {
    var node = this.dataDoc.createTextNode("\n"); //xxx
    aRootNode.appendChild(node);
  },

  /**** real job is done here *************************************************/
  getAppName: function() {
    var appName = ILHelpers.hostApp;
    for(i in ILHelpers.AppGuids)
      if(appName == ILHelpers.AppGuids[i])
        return i;
    return "";
  },

  // this is capable of creating the following nodes: "lastupd", "useragent"
  appendNode: function(aType) {
    var data;
    if(aType == "lastupd") { // current date/time, GMT
      var date = new Date();
      data = date.toGMTString();
    } else if(aType == "useragent") { // ua, possibly modified by others
      data = navigator.userAgent;
    }
    var node = this.makeNodeWithText(aType, data);
    this.infoElt.appendChild(node);
    this.insertLinebreak(this.infoElt);
  },

  // addonsList() returns the list of themes or extensions, depending on aTarget
  //   aTarget must be "extension" or "theme" (case-sensitive)
  // original code by Torisugari

  // available extensions.rdf arcs
  emAttributes: {name: "", version: "", disabled: "", internalName: "",
                 homepageURL: "", description: "", creator: ""},
  appendAddonsList: function(aTarget) // aTarget must be "extension" or "theme"
  {
    function getValue(aArcName) {
      var target = extensionDS.GetTarget(element, emArcs[aArcName], true);
      if(target) return ILHelpers.QI(target, "nsIRDFLiteral").Value;
      return "";
    }

    const XMLNS_EM = "http://www.mozilla.org/2004/em-rdf#";
    var rdfService = ILHelpers.getService("rdf/rdf-service;1", "nsIRDFService");
    var container = ILHelpers.getService("rdf/container;1", "nsIRDFContainer");
    var extensionDS = ILHelpers.getService("extensions/manager;1",
      "nsIExtensionManager").datasource;
      
    // get the enumerator of all addons of type aTarget --> |elements|
    var root = rdfService.GetResource("urn:mozilla:" + aTarget + ":root");
    container.Init(extensionDS, root);
    var elements = container.GetElements();

    // RDF arcs available for each addon
    var attr, emArcs = [];
    for(attr in this.emAttributes)
      emArcs[attr] = rdfService.GetResource(XMLNS_EM + attr);

    //
    var nodeName;
    if(aTarget == "extension")
      nodeName = "ext";
    else if(aTarget == "theme")
      nodeName = "theme";

    // -------- start collecting information --------
    var rootNode = this.dataDoc.createElement(aTarget + "s");
    this.infoElt.appendChild(rootNode);
    this.insertLinebreak(this.infoElt);
    this.insertLinebreak(rootNode);

    // remember selected theme
    if(aTarget == "theme") {
      try {
        var selectedTheme = ILPrefs.prefSvc.getCharPref("general.skins.selectedSkin");
      } catch(e) {}
    }

    // enumerate all addons of given type
    while(elements.hasMoreElements())
    {
      var element = elements.getNext();
      try {
        ILHelpers.QI(element, "nsIRDFResource");

        // we put the name in a text node inside the elt.
        var addonName = getValue("name");
        var addon = this.makeNodeWithText(nodeName, addonName);

        // read RDF data to |addon|
        for (attr in this.emAttributes) {
          var val = getValue(attr);
          if((val != "") && (attr != "name")) // "name" is handled above
            addon.setAttribute(attr, val);
        }
        //id = element.Value;
        rootNode.appendChild(addon);
        this.insertLinebreak(rootNode);

        if(aTarget == "theme") {
          if(addon.getAttribute("internalName") == selectedTheme)
            addon.setAttribute("selected", "true");
        } else {
          // a dirty fix for someone else's mistake because of which DOMi has
          // a hundred of entries in extensions.rdf in people's profiles
          if(this.domifix && (name == "DOM Inspector")) {
            if(this.DOMiListed)
              continue;
            this.DOMiListed = true;
          }
        }
      } catch(e) {
        ILHelpers.dumpException(e);
      }
    }
  },


  // return all browser plugins
  appendPlugins: function() {
    var rootNode = this.dataDoc.createElement("plugins");
    this.insertLinebreak(rootNode);

    var plugins = [];
    try {
      navigator.plugins.refresh(false);
      var numPlugins = navigator.plugins.length;
      for (var i=0; i<numPlugins; i++)
      {
        var plugin = navigator.plugins[i];
        if (plugin)
        {
          var title = plugin.name;
          var duplicate = false;
          for(var j=0; j<plugins.length; j++) { // yes it's O(n^2). Kill me.
            if(plugins[j] == title) {
              duplicate = true;
              break;
            }
          }

          //plugin.filename;
          //plugin.description;
          if(!duplicate) {
            var node = this.makeNodeWithText("plugin", title);
            rootNode.appendChild(node);
            plugins.push(title);
            this.insertLinebreak(rootNode);
          }
        }
      }
    } catch(e) {}
    
    if(plugins.length > 0) {
      this.infoElt.appendChild(rootNode);
      this.insertLinebreak(this.infoElt);
    }
  }
};

