/* common.js
 * Defines: several utility objects:
 *  1) an object that returns a working InfoLister instance [InfoListerGlue]
 *  2) "windows manager" [InfoListerWindows]
 *  3) various helpers [ILHelpers]
 *
 * Part of InfoLister by Nickolay Ponomarev <asqueella@gmail.com>
 * Copyright (c) 2004-2005, Nickolay Ponomarev.
 * See the license.txt included in this package for more information.
 */

// We use this object to get a reference to a working InfoLister instance from
// a window that doesn't include infolister.js & friends script by default.
var InfoListerGlue = {
  getInfoLister: function(aWindow) {
    // Note: |window| is not defined when this is used from the about:info 
    // component. We get an |aWindow| parameter which is |window|, when it's
    // available and |undefined|, when it's not available. Loading InfoLister
    // to the same scope implementation component does not work (since we use
    // document.implementation, for example), so we assume there is a browser
    // window that called component's newChannel method, so we want to load IL 
    // there instead.
    // xxx current temporary fix is to always load IL in browser windows, so 
    // that we never get to the step 3 when called from component.
    
    // First check if we have IL loaded in current window already
    if(aWindow && ("InfoLister" in aWindow)) return aWindow.InfoLister;
    
    // Second, enumerate other windows to see if IL object is loaded in any of them
    var mediator = ILHelpers.getService("appshell/window-mediator;1", "nsIWindowMediator");
    var enumerator = mediator.getEnumerator("");
    while(enumerator.hasMoreElements())
    {
      var win = enumerator.getNext();
      if("InfoLister" in win)
        return win.InfoLister;
      if(!aWindow)
        aWindow = win;
    }

    if(!aWindow) throw "InfoLister: shouldn't get here";
    
    // As a last resort, try load the needed scripts in current window manually
    try {
      var loader = ILHelpers.createInstance("moz/jssubscript-loader;1", "mozIJSSubScriptLoader");
      var objToLoad = {
        ILFormatter: "format.js",
        ILInfoCollector: "collect.js",
        InfoLister: "infolister.js"
      };
      for(var obj in objToLoad) {
        if(!(obj in aWindow))
          loader.loadSubScript("chrome://infolister/content/" + objToLoad[obj]);
      }
    } catch(e) {
      ILHelpers.dumpException(e);
      throw e;
    }
    return InfoLister;
  }
}

/******************************************************************************/
var InfoListerWindows = {
  // windows we work with
  Info: 0, Prefs: 1, Browser: 2, Mail: 3,

  urls: ["chrome://infolister/content/infolister.xul",
         "chrome://infolister/content/prefs.xul",
         "chrome://browser/content/browser.xul",
         "chrome://messenger/content/messenger.xul"],
  winTypeStrings: ["infolister:infowindow", "infolister:prefwindow",
  "navigator:browser", ""],

  open: function(aWindowType) {
    var win = this.get(aWindowType);
    if(win)
      win.focus();
    else
      win = window.open(this.urls[aWindowType], "", "chrome,resizable");

    return win;
  },

  // return nsIDOMWindow if a window of given type exists, null otherwise
  get: function(aWindowType) {
    var windowmanager = ILHelpers.getService(
          "appshell/window-mediator;1", "nsIWindowMediator");
    var win;
    if(aWindowType == this.Mail) { // damn Mail window doesn't have a windowtype
      var enumerator = windowmanager.getEnumerator("");
      while(enumerator.hasMoreElements())
      {
        win = enumerator.getNext();
        if(win.InfoLister) return win;
      }
    } else
      win = windowmanager.getMostRecentWindow(this.winTypeStrings[aWindowType]);
    return win ? win : null;
  },

  startUpload: function(aFileName) {
    window.openDialog("chrome://infolister/content/upload.xul",
      "ilupload", "chrome", InfoLister.getFileByPath(aFileName));
  }
};

/******************************************************************************/
var ILHelpers = {
  // xpcom helpers
  createInstance: function(aCID, aIID) {
    return Components.classes["@mozilla.org/"+aCID]
          .createInstance(Components.interfaces[aIID]);
  },

  getService: function(aCID, aIID) {
    return Components.classes["@mozilla.org/"+aCID]
          .getService(Components.interfaces[aIID]);
  },
  
  QI: function(aObj, aIID) {
    return aObj.QueryInterface(Components.interfaces[aIID]);
  },

  // debug
  _console: null,
  dumpException: function(e) {
    var msg = "unexpected exception. dev info follows:\n";
    for (var i in e)
      msg += (i + "=" + e[i] + "; ");
    this.dump(msg);
  },

  dump: function(msg) {
    if(!this._console)
      this._console = this.getService("consoleservice;1", "nsIConsoleService");
    this._console.logStringMessage("infolister: " + msg);
  },

  // misc  
  AppGuids: {
    Firefox: "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}",
    Thunderbird: "{3550f703-e582-4d05-9a08-453d09bdfdc6}"
  },
  get hostApp() {
    // In post 1.0 builds nsIXULAppInfo provides application info
    if ("@mozilla.org/xre/app-info;1" in Components.classes)
      return ILHelpers.getService("xre/app-info;1", "nsIXULAppInfo").ID.toString();

    // in Fx/Tb 1.0 a pref is used
    return ILPrefs.prefSvc.getCharPref("app.id");
  }, 
  
  pickFile: function(aTitle, aDefaultFile)
  { // show filepicker, return selected file name.
    var fp = ILHelpers.createInstance("filepicker;1", "nsIFilePicker");
    fp.init(window, aTitle, fp.modeSave);
    
    if(aDefaultFile) {
      fp.displayDirectory = aDefaultFile.parent;
      fp.defaultString = aDefaultFile.leafName;
    }

    var filters = { filterText: 0, filterHTML: 1, filterXML: 2, filterAll: 3 };
    for(var i in filters)
      fp.appendFilters(fp[i]);
    fp.filterIndex = ILPrefs.getIntPref("ouputmode");

    if (fp.show() == fp.returnCancel)
      return null;

    if(fp.filterIndex == 3)
      fp.filterIndex = ILPrefs.getIntPref("ouputmode");

    // xxx Add an extension to the file depending on filter - find a moz bug on filepicker

    return { file: fp.file, mode: fp.filterIndex };
  }
};

var IL_Loader = ILHelpers.createInstance("moz/jssubscript-loader;1", "mozIJSSubScriptLoader");
IL_Loader.loadSubScript("chrome://infolister/content/prefutils.js");
IL_Loader = undefined; // ?

var ILPrefs = new PrefsWrapper1("extensions.infolister.");

